function [EST, STRUCT] = FractalConnectTest(data1,data2,Nwt, alpha, j1, j2, FIG);
% 
%% function [EST, STRUCT] = FractalConnectTest(data1,data2,Nwt, alpha, j1, j2, FIG);
% 
% ---- FISHER TRANSFORM BASED TEST FOR FRACTAL CONNECTIVITY ----
% INPUT:  -----------------------
%   data1, data2  - input data
%   Nwt           - wavelet (Daubechies') vanishing moments
%   alpha         - significance level (e.g. alpha=0.1)
%   j1, j2        - scales over which test is performed
%   FIG           - (optional) Figure number
% OUTPUT: -----------------------
%   EST         - test and estimates
%               fields:
%                   dec         - test decision
%                   pval        - test statistic p value
%                   hatpower    - test power estimate (from non central chi2)
%                   T_theo      - test statistic value
%                   seuil       - test critical value
%                   Co          - mean over scales j1:j2 of correlation coefficient estimate
%                   aX          - exponent d1
%                   aY          - exponent d2
%                   aXY         - exponent d34
%   STRUCT      - structure functions
%               fields:
%                   RR          - correlation coefficient estimate at each scale
%                   Zxy         - Fisher transformed correlation coefficient  at each scale
%                   Exx         - mean( d_X(j,.)^2 ) structure functions vector of X (wavelet auto-spectra)
%                   Eyy         - mean( d_Y(j,.)^2 ) structure functions vector of Y
%                   Exy         - mean( d_X(j,.).*d_X(j,.) ) structure functions vector (wavelet inter-spectrum)
%                   NJ          - number of coefficients per scale
%
%   NOTE: The function uses the inline function rlistcoefdaub.m by Patrice Abry (C 1995)
%
% Herwig Wendt, Lyon, Peyresq, 2008. Patrice Abry. Lyon, 1995
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%% For details on the methods, please refer to the related paper:
%
%     Herwig Wendt, Antoine Scherrer, Patrice Abry, Sophie Achard, 
%     "Testing fractal connectivity in multivariate long memory processes," 
%     Proc. IEEE Int. Conf. Acoust., Speech, and Signal Proc. (ICASSP), Taipei, Taiwan, 2009.
%
%% If you use this code, please quote this reference.
%
%         @INPROCEEDINGS{Wendt2009,
%           author = {Herwig Wendt and Antoine Scherrer and Patrice Abry and Sophie Achard},
%           title = {Testing fractal connectivity in multivariate long memory processes},
%           booktitle = {Proc. IEEE Int. Conf. Acoust., Speech, and Signal Proc. (ICASSP)},
%           address = {Taipei, Taiwan},
%           year = {2009}}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin<7; FIG=0; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet auto-/inter-spectra and  coherence function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

loge = log2(exp(1)) ;

nj = length(data1) ;
h1 = rlistcoefdaub(Nwt) ;
nl = length(h1) ;
g1 = (-1).^(0:-1+nl).*fliplr(h1) ;
gg1 = fliplr(g1) ;
hh1 = fliplr(h1) ;
lrhoj = 1;
jmax=fix(log2(nj/(2*Nwt+1)));

for j=1:jmax,
         conv1=conv(data1,gg1) ;
         conv2=conv(data2,gg1) ;
         dec1=conv1(nl:2:nj-1) ;
         dec2=conv2(nl:2:nj-1) ;
         Exx(j) = mean(abs(dec1).*abs(dec1)) ;
         Eyy(j) = mean(abs(dec2).*abs(dec2)) ;
         Exy(j) = mean(dec1.*dec2) ;
         NJ(j)=length(dec1) ;
         rhoj(j) = Exy(j) ./ sqrt(Exx(j).*Eyy(j));
         clear conv1 dec1 conv2 dec2
         conv1  =conv(data1,hh1) ;
         conv2  =conv(data2,hh1) ;
         data1 = conv1(nl:2:nj-1) ;
         data2 = conv2(nl:2:nj-1) ;
         nj = length(data1) ;
         Vj1(j) = loge^2 * 2 / NJ(j);
         Vj2(j) = loge^2 * 2 / NJ(j);
         if NJ(j) >= 2^12
             lrhoj = rhoj(j);
             Vij(j) = loge^2 / NJ(j) * (1 + 1/rhoj(j)^2);
             Vij_cov(j) = loge^2 / NJ(j) * (1/rhoj(j)^2 + rhoj(j)^2 - 2);
         else
             Vij(j) = loge^2 / NJ(j) * (1 + 1/lrhoj^2);
             Vij_cov(j) = loge^2 / NJ(j) * (1/lrhoj^2 + lrhoj^2 - 2);
         end
end

index=find(NJ>=2);
Exx=Exx(index) ;
Eyy=Eyy(index) ;
Exy=Exy(index) ;
NJ=NJ(index) ;
Vj1=Vj1(index) ;
Vj2=Vj2(index) ;
Vij=Vij(index);
Vij_cov=Vij_cov(index);

coefcorr = Exy ./ sqrt(Exx.*Eyy) ;

j2=min(j2,length(NJ));

jj = j1:j2 ;
njj  = NJ(jj);
J  = length(NJ); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fisher Test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Calculate Fisher's Z statistic
    for jid=1:J
        RR(jid)=Exy(jid)/sqrt(Exx(jid)*Eyy(jid));                  
        Zxy(jid)=1/2*log( (1+RR(jid)) / (1-RR(jid)) );        
    end
Co=mean(RR);
S_theo=1./sqrt(NJ-3);

%% calculate UMPI test of equality of means of Gaussian RVs
Mcommon=sum(Zxy(jj)./S_theo(jj).^2,2);
Scommon_theo=sum(1./S_theo(jj).^2,2);
Mcommon_theo=Mcommon./Scommon_theo;  % common mean from theo variances
TMP_theo=1./S_theo(jj).^2.*(Zxy(jj)-repmat(Mcommon_theo,1,length(jj))).^2;

T_theo=sum(TMP_theo);
seuil=chi2inv(1-alpha, length(jj)-1);

pval=1-(chi2cdf(T_theo,length(jj)-1));
dec=1.0*(T_theo>=seuil);


%% calculate exponents
wvarjj = 1./njj;
S0 = sum(1./wvarjj) ;
S1 = sum(jj./wvarjj) ;
S2 = sum(jj.^2./wvarjj) ;
wjj = (S0 * jj - S1) ./ wvarjj / (S0*S2-S1*S1) ;
vjj = (S2 - S1 .* jj) ./ wvarjj ./ (S0.*S2-S1.*S1) ;
aX  =sum(wjj .* log2(Exx(jj)))/2;       aXaest  =sum(vjj .* log2(Exx(jj)))/2;
aY  =sum(wjj .* log2(Eyy(jj)))/2;       aYaest  =sum(vjj .* log2(Eyy(jj)))/2;
aXY  =sum(wjj .* log2(abs(Exy(jj))));   aXYaest  =sum(vjj .* log2(Exy(jj)));


%% noncentral chi2 power


ZxyP=1/2*log( (1+RR(jj)) ./ (1-RR(jj)) );    
McommonP=sum(ZxyP./(S_theo(jj)).^2);
ScommonP_theo=sum(1./S_theo(jj).^2);
McommonP_theo=McommonP./ScommonP_theo;
TMP_theo=1./(S_theo(jj)).^2.*(ZxyP-McommonP_theo).^2;
TP_theo=sum(TMP_theo);

P=1-ncx2cdf(seuil, length(jj)-1, TP_theo);

hatpower=P;

EST.dec=dec;
EST.pval=pval;
EST.hatpower=hatpower;
EST.T_theo=T_theo;
EST.seuil=seuil;
EST.Co=Co;
EST.aX=aX;
EST.aY=aY;
EST.aXY=aXY;

STRUCT.RR=RR;
STRUCT.Zxy=Zxy;
STRUCT.Exx=Exx;
STRUCT.Eyy=Eyy;
STRUCT.Exy=Exy;
STRUCT.NJ=NJ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if FIG>0
    figure(ceil(FIG)); clf;
    x=linspace(0,max(30, T_theo),100);
    plot(x, chi2pdf(x,length(jj)-1), 'k'); hh=chi2pdf(x,length(jj)-1); hold on;
    plot([seuil seuil], [0 max(hh)], 'k');
    plot([T_theo T_theo], [0 max(hh)*1.1], 'r');
    plot(x, ncx2pdf(x, length(jj)-1, mean(TP_theo)), 'b--');
    hold off; grid on;
    text('string',sprintf('Alpha = %2.1f %% - rejected = %g  pvalue= %1.2f   (PowerEst=%2.1f %%)\n',alpha*100,dec,pval,P*100),'units','normalized','pos',[0.05 0.9]);
    text('string',sprintf('Coest = %2.1f %% - T=%2.3e - FISHER Z test j=[%g;%g] \n',mean(Co)*100,mean(T_theo),j1,j2),'units','normalized','pos',[0.05 0.85]);

    figure(ceil(FIG)+1); clf;
    subplot(131); hold on;
    plot(1:jmax, log2(Exx),'r.-'); 
    reg1=polyval([2*aX,2*aXaest],1:jmax); plot(1:jmax, reg1,'k--');
    reg2=polyval([2*aX,2*aXaest],j1:j2); plot(j1:j2, reg2,'k-');
    set(gca,'xlim',[0.5 jmax+0.5]); 
    xlabel('j'); ylabel('log_2(S_X(2^j))'); title(['\alpha_X=',num2str(aX)]);
    grid on; hold off;
    subplot(132); hold on;
    plot(1:jmax, log2(Eyy),'r.-'); 
    reg1=polyval([2*aY,2*aYaest],1:jmax); plot(1:jmax, reg1,'k--');
    reg2=polyval([2*aY,2*aYaest],j1:j2); plot(j1:j2, reg2,'k-');
    set(gca,'xlim',[0.5 jmax+0.5]); 
    xlabel('j'); ylabel('log_2(S_Y(2^j))'); title(['\alpha_Y=',num2str(aY)]);
    grid on; hold off;
    subplot(133); hold on;
    plot(1:jmax, log2(Exy),'r.-'); 
    reg1=polyval([aXY,aXYaest],1:jmax); plot(1:jmax, reg1,'k--');
    reg2=polyval([aXY,aXYaest],j1:j2); plot(j1:j2, reg2,'k-');
    set(gca,'xlim',[0.5 jmax+0.5]); 
    xlabel('j'); ylabel('log_2(S_{XY}(2^j))'); title(['\alpha_{XY}=',num2str(aXY)]);
    grid on; hold off;
    
    figure(ceil(FIG)+2); clf;
    subplot(121); hold on;
    plot(1:jmax, log2(abs(RR)),'r.-'); 
    aR  =sum(wjj .* log2(abs(RR(jj))));   aRaest  =sum(vjj .* log2(RR(jj)));
    reg1=polyval([aR,aRaest],1:jmax); plot(1:jmax, reg1,'k--');
    reg2=polyval([aR,aRaest],j1:j2); plot(j1:j2, reg2,'k-');
    set(gca,'xlim',[0.5 jmax+0.5]); set(gca,'ylim',[floor(min(abs(log2(RR))))-1 ceil(max(abs(log2(RR))))+1]); 
    xlabel('j'); title('log_2(|\gamma_{XY}(2^j)|)'); 
    grid on; hold off;
    subplot(122); hold on;
    plot(1:jmax, Zxy,'r.-'); 
    set(gca,'xlim',[0.5 jmax+0.5]); set(gca,'ylim',[floor(min(Zxy)) ceil(max(Zxy))]); 
    xlabel('j'); title('z_{XY}(2^j)'); grid on; hold off;
end



    function h=rlistcoefdaub(regu);
        if ~(regu<11 & regu>0),
            mess=[' Ondelette non implantee'];
            disp(mess);
            return;
        else
            if regu==1,
                h=[1/sqrt(2) 1/sqrt(2)];
                %  g=[1/sqrt(2) -1/sqrt(2)];
            end
            if regu==2,
                h(1:2)=[0.482962913145 0.836516303738];
                h(3:4)=[0.224143868042 -0.129409522551];
            end
            if regu==3,
                h(1:2)=[0.332670552950 0.806891509311];
                h(3:4)=[0.459877502118 -0.135011020010];
                h(5:6)=[-0.085441273882 0.035226291882];
            end
            if regu==4,
                h(1:2)=[0.230377813309 0.714846570553];
                h(3:4)=[0.630880767930 -0.027983769417];
                h(5:6)=[-0.187034811719 0.030841381836];
                h(7:8)=[0.032883011667 -0.010597401785];
            end
            if regu==5,
                h(1:2)=[0.160102397974 0.603829269797];
                h(3:4)=[0.724308528438 0.138428145901];
                h(5:6)=[-0.242294887066 -0.032244869585];
                h(7:8)=[0.077571493840 -0.006241490213];
                h(9:10)=[-0.012580751999 0.003335725285];
            end
            if regu==6,
                h(1:2)=[0.111540743350 0.494623890398];
                h(3:4)=[0.751133908021 0.315250351709];
                h(5:6)=[-0.226264693965 -0.129766867567];
                h(7:8)=[0.097501605587 0.027522865530];
                h(9:10)=[-0.031582039318 0.000553842201];
                h(11:12)=[0.004777257511 -0.001077301085];
            end
            if regu==7,
                h(1:2)=[0.077852054085 0.396539319482];
                h(3:4)=[0.729132090846 0.469782287405];
                h(5:6)=[-0.143906003929 -0.224036184994];
                h(7:8)=[0.071309219267 0.080612609151];
                h(9:10)=[-0.038029936935 -0.016574541631];
                h(11:12)=[0.012550998556 0.000429577973];
                h(13:14)=[-0.001801640704 0.000353713800];
            end
            if regu==8,
                h(1:2)=[0.054415842243 0.312871590914];
                h(3:4)=[0.675630736297 0.585354683654];
                h(5:6)=[-0.015829105256 -0.284015542962];
                h(7:8)=[0.000472484574 0.128747426620];
                h(9:10)=[-0.017369301002 -0.044088253931];
                h(11:12)=[0.013981027917 0.008746094047];
                h(13:14)=[-0.004870352993 -0.000391740373];
                h(15:16)=[0.000675449406 -0.000117476784];
            end
            if regu==9,
                h(1:2)=[0.038077947364 0.243834674613];
                h(3:4)=[0.604823123690 0.657288078051];
                h(5:6)=[0.133197385825 -0.293273783279];
                h(7:8)=[-0.096840783223 0.148540749338];
                h(9:10)=[0.030725681479 -0.067632829061];
                h(11:12)=[0.000250947115 0.022361662124];
                h(13:14)=[-0.004723204758 -0.004281503682];
                h(15:16)=[0.001847646883 0.000230385764];
                h(17:18)=[-0.000251963189 0.000039347320];
            end
            if regu==10,
                h(1:2)=[0.026670057901 0.188176800078];
                h(3:4)=[0.52720118932 0.688459039454];
                h(5:6)=[0.281172343661 -0.249846424327];
                h(7:8)=[-0.195946274377 0.127369340336];
                h(9:10)=[0.093057364604 -0.071394147166];
                h(11:12)=[-0.029457536822 0.033212674059];
                h(13:14)=[0.003606553567 -0.010733175483];
                h(15:16)=[0.001395351747 0.001992405295];
                h(17:18)=[-0.000685856695 -0.000116466855];
                h(19:20)=[0.000093588670 -0.000013264203];
            end
        end
    end
end